/*
    Copyright 2004,2005,2006 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef WIN32
    #define strncasecmp strnicmp
    #define u_char      unsigned char
#endif



#define VER         "0.1"
#define FILENAME    "popcorn.ini"
#define PROF        "ProfileName="
#define ADDR        "Address="
#define USER        "UserName="
#define REMPWD      "RememberPassword="
#define PASS        "Password="
#define BUFFSZ      1024



u_char *popcorn_dec(u_char *data);
void std_err(void);



int main(int argc, char *argv[]) {
    FILE    *fd;
    u_char  buff[BUFFSZ + 1],
            *ptr;

    setbuf(stdout, NULL);

    fputs("\n"
        "Ultrafunk Popcorn passwords decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n"
        "Usage:\n"
        "------\n"
        "Without parameters the program automatically retrieves all the passwords from the Popcorn configuration file "FILENAME", but if you provide an encoded password at command-line it will be decoded and showed\n"
        "\n", stdout);

    if(argc > 1) {
        printf("\nPassword: %s\n", popcorn_dec(argv[1]));
        return(0);
    }

    fputs("\n- Opening popcorn configuration file: "FILENAME"\n", stdout);
    fd = fopen(FILENAME, "rb");
    if(!fd) std_err();

    while(1) {
        if(!fgets(buff, BUFFSZ, fd)) break;

        if(!strncasecmp(buff, PROF, sizeof(PROF) - 1)) {
            printf("\n"
                "------------\n"
                "Profile:    %s",
                buff + sizeof(PROF) - 1);

        } else if(!strncasecmp(buff, ADDR, sizeof(ADDR) - 1)) {
            printf("E-mail:     %s",
                buff + sizeof(ADDR) - 1);

        } else if(!strncasecmp(buff, USER, sizeof(USER) - 1)) {
            printf("User:       %s",
                buff + sizeof(USER) - 1);

        } else if(!strncasecmp(buff, REMPWD, sizeof(REMPWD) - 1)) {
            if(strncasecmp(buff + sizeof(REMPWD) - 1, "TRUE", 4)) {
                fputs("- Password is NOT stored!\n", stdout);
            }

        } else if(!strncasecmp(buff, PASS, sizeof(PASS) - 1)) {
            ptr = strchr(buff, 0x0d);
            if(!ptr) {
                ptr = strchr(buff, 0x0a);
                if(ptr) *ptr = 0;
            } else *ptr = 0;

            ptr = buff + sizeof(PASS) - 1;
            printf(
                "Password:   %s\n"
                "   encoded: %s\n",
                popcorn_dec(ptr),
                ptr);
        }
    }

    fclose(fd);
    fputs("\n"
        "------------\n"
        "    Press RETURN to exit\n", stdout);
    fgetc(stdin);
    return(0);
}



u_char *popcorn_dec(u_char *data) {
    static  u_char  buff[4096];
    u_char          eax,
                    *out;
    int             edi,
                    edx;

    out = buff;
    eax = *data++;
    if(eax) {
        edi = 0x193BAA1C;
        while(1) {
            edx = eax + 0x0FFFFFBF;
            eax = *data++;
            if(!eax) break;
            eax += (edx << 4) - 'A';
            *out++ = eax ^ (edi >> 0xf);
            edi *= 0x00B504F3;
            eax = *data++;
            if(!eax) break;
            edi++;
        }
    }
    *out = 0;
    return(buff);
}



void std_err(void) {
    perror("\nError");
    fputs("\n    Press RETURN to exit\n", stdout);
    fgetc(stdin);
    exit(1);
}


